-----------------------------------------------------------------------
-- GNOMECHARMER
-- Charm your friends with your very own raid icon tagger, the gnome way!
--
-- Combines some of the features of ezIcons and JasonTag into one tiny
-- package. Cleaned up and tuned up for your pleasure!
--
-- RADIAL
--
-- The first part of the addon is the radial menu. Whenever you double-click a
-- mob or a friendly player in the game world, you'll get a circle (radial menu)
-- with all the raid target icons around the cursor. If the target does not have
-- a raid target icon, moving your mouse over any of them will assign that icon
-- to the target. If the target is marked, moving your mouse over that icon will
-- un-mark the target. Clicking with your mouse inside the circle will dismiss
-- the radial menu.
--
-- TAG BAR
--
-- The second part of the addon is the tag bar. It should show itself when you
-- are grouped and the raid/party leader.
--  * Click to the far right on the bar to drag and move it around.
--  * Target a mob and click any of the icons to toggle marking that target with
--    the selected icon.
--  * Click an icon and drag'n'drop it on a target to mark that mouseover target
--    with the selected icon.
--
-- You can not turn off either of the features from in-game, you'll have to
-- comment out the code right here.
--
-- DIMMED ICONS
--
-- The addon also tries to keep a live list of raid target icons used (only
-- those set by yourself can be trusted, it doesn't scan targets) and dim out
-- any icons used already so that the unused ones are more prominent in the bar
-- and radial menu.
--
-- rabbit.magtheridon@gmail.com
--

local iconsUsed = {}

local function validate(target)
	if not UnitExists(target) then return end
	local can = true
	if not (GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0) or (not IsPartyLeader() and not IsRaidOfficer()) then
		can = nil
	end
	if can and not UnitIsUnit("player", target) then
		if not UnitPlayerOrPetInParty(target) and not UnitPlayerOrPetInRaid(target) then
			if UnitIsPlayer(target) and not UnitCanCooperate("player", target) then
				can = nil
			end
		end
	end
	return can
end

local function setIcon(target, id)
	if not validate(target) then return end
	local current = GetRaidTargetIndex(target)
	if current then iconsUsed[current] = nil end
	if current and id == current then id = 0 end
	SetRaidTarget(target, id)
end

-----------------------------------------------------------------------
-- Radial menu
--
-- Most of the code is ripped from ezIcons. Thanks!
--

local radialIcons = {}
do
	local menu = nil
	local function update()
		local x, y = GetCursorPosition()
		local s = menu:GetEffectiveScale()
		local mx, my = menu:GetCenter()
		local a, b = (y / s) - my, (x / s) - mx
		local dist = math.floor(math.sqrt(a * a + b * b))
		if dist > 20 then
			if dist < 50 then
				local pos = math.deg(math.atan2(a, b)) + 27.5
				local iconIndex = mod(11 - ceil(pos / 45), 8) + 1
				setIcon("target", iconIndex)
			end
			this:Hide()
		end
	end
	local function hide()
		this:Hide()
	end

	local function createRadial()
		menu = CreateFrame("Button", "GnomeCharmerRadial", UIParent)
		menu:SetWidth(100)
		menu:SetHeight(100)
		menu:SetPoint("CENTER", UIParent, "BOTTOMLEFT", 0, 0)
		menu:RegisterForClicks("LeftButtonUp", "RightButtonUp")
		menu:RegisterEvent("PLAYER_TARGET_CHANGED")
		menu:SetScript("OnUpdate", update)
		menu:SetScript("OnEvent", hide)
		menu:SetScript("OnClick", hide)
		for i = 1, 8 do
			local icon = menu:CreateTexture("GnomeCharmerRadial"..i, "OVERLAY")
			local radians = (0.375 - i / 8) * 360
			icon:SetPoint("CENTER", menu, "CENTER", 36 * cos(radians), 36 * sin(radians))
			icon:SetWidth(18)
			icon:SetHeight(18)
			icon:SetAlpha(iconsUsed[i] and 0.2 or 1)
			icon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
			SetRaidTargetIconTexture(icon, i)

			radialIcons[i] = icon
		end
	end

	local function showRadial()
		if not validate("target") then return end
		if not menu then createRadial() end

		local x,y = GetCursorPosition()
		local s = menu:GetEffectiveScale()
		menu:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x/s, y/s)
		menu:Show()
	end

	local doubleClick, doubleClickX, doubleClickY = nil, 0, 0
	local orig = WorldFrame:GetScript("OnMouseUp")
	WorldFrame:SetScript("OnMouseUp", function(self, button)
		if button == "LeftButton" then
			local time = GetTime()
			local x, y = GetCursorPosition()
			if doubleClick and time - doubleClick < 0.25 and math.abs(x - doubleClickX) < 20 and math.abs(y - doubleClickY) < 20 then
				showRadial()
				doubleClick = nil
			else
				doubleClick = time
			end
			doubleClickX, doubleClickY = x, y
		end
		if orig then orig() end
	end)
end

-----------------------------------------------------------------------
-- Tag bar
--
-- Mostly ninjaed from JasonTag. Thanks!
--

local barIcons = {}
do
	local bar = nil

	local tt = {
		"Target someone and |cffeda55fClick|r an icon, or |cffeda55fdrag and drop|r an icon on someone to mark them. Use the left icon on a target to clear its mark.",
		"|cffeda55fDouble-Click|r on targets in the world to get the radial icon selector up.",
	}
	local function tooltip()
		if InCombatLockdown() then return end
		GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
		GameTooltip:AddLine("GnomeCharmer")
		for i, v in pairs(tt) do
			GameTooltip:AddLine(v, 1, 1, 1, 1)
			GameTooltip:AddLine(" ")
		end
		GameTooltip:AddLine("Click and drag the right edge of the bar to move it.", .5, .5, .5, 1)
		GameTooltip:Show()
	end
	local function clearTooltip()
		GameTooltip:Hide()
	end

	local function onUpdate()
		local x, y = GetCursorPosition()
		local s = bar:GetEffectiveScale()
		this:ClearAllPoints()
		this:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", (x / s ) + 20, (y / s) - 35)
	end
	local function onDragStart()
		this:SetScript("OnUpdate", onUpdate)
	end
	local function onDragStop()
		this:SetScript("OnUpdate", nil)
		this:GetScript("OnMouseUp")()
	end
	local function onClick()
		setIcon("target", this:GetID())
	end
	local function onMouseUp()
		local i = this:GetID()
		this:ClearAllPoints()
		this:SetPoint("TOPLEFT", (i * 18) + 4, -4)
		this:SetPoint("CENTER")
		setIcon("mouseover", i)
	end
	local function createButton(frame, texture, id)
		local name = frame:GetName()
		local button = CreateFrame("Button", name .. "Button" .. id, frame)
		button:SetMovable(true)
		button:SetWidth(18)
		button:SetHeight(18)
		button:SetPoint("TOPLEFT", (id * 18) + 4, -4)
		button:SetID(id)

		button:RegisterForDrag("LeftButton")
		button:SetScript("OnDragStart", onDragStart)
		button:SetScript("OnDragStop", onDragStop)
		button:SetScript("OnMouseUp", onMouseUp)
		button:SetScript("OnClick", onClick)
		button:SetScript("OnEnter", tooltip)
		button:SetScript("OnLeave", clearTooltip)

		local icon = button:CreateTexture(name .. "Button" .. id .. "Icon", "ARTWORK")
		icon:SetTexture(texture)
		icon:SetAllPoints()
		if id > 0 then
			SetRaidTargetIconTexture(icon, id)
		end
		return icon
	end
	local function barOnMouseDown()
		this:StartMoving()
	end
	local function barOnHide()
		this:StopMovingOrSizing()
	end
	local function barOnEvent()
		if GnomeCharmerDB.barShow and (GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0) and (IsPartyLeader() or IsRaidOfficer()) then
			bar:Show()
		else
			bar:Hide()
		end
	end
	local function barOnMouseWheel(self, direction)
		local a = direction > 0 and this:GetAlpha() - 0.1 or this:GetAlpha() + 0.1
		GnomeCharmerDB.barAlpha = a
		this:SetAlpha(a)
	end
	local function createBar()
		local frame = CreateFrame("Button", "GnomeCharmerBar", UIParent)
		frame:SetWidth(180)
		frame:SetHeight(28)
		frame:SetClampedToScreen(true)
		frame:SetFrameStrata("LOW")
		frame:SetToplevel(1)
		frame:SetAlpha(GnomeCharmerDB and GnomeCharmerDB.barAlpha or 1)
		frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 150, -150)
		frame:SetBackdrop({
			bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
			edgeSize = 8,
			insets = {left = 2, right = 2, top = 2, bottom = 2}
		})
		frame:SetBackdropBorderColor(1, 1, 1)
		frame:SetBackdropColor(0.094, 0.094, 0.094)

		frame:SetMovable(true)
		frame:RegisterForDrag("LeftButton")
		frame:EnableMouseWheel(1)
		frame:SetScript("OnMouseWheel", barOnMouseWheel)
		frame:SetScript("OnDragStart", barOnMouseDown)
		frame:SetScript("OnDragStop", barOnHide)
		frame:SetScript("OnHide", barOnHide)
		frame:SetScript("OnEnter", tooltip)
		frame:SetScript("OnLeave", clearTooltip)

		createButton(frame, "Interface\\Buttons\\UI-GroupLoot-Pass-Up", 0)
		for i = 1, 8 do
			barIcons[i] = createButton(frame, "Interface\\TargetingFrame\\UI-RaidTargetingIcons", i)
		end

		frame:RegisterEvent("PARTY_MEMBERS_CHANGED")
		frame:RegisterEvent("PARTY_LEADER_CHANGED")
		frame:SetScript("OnEvent", barOnEvent)

		bar = frame
	end

	-- Keybinding for toggline the bar on and off
	BINDING_HEADER_GNOMECHARMER = "GnomeCharmer"
	BINDING_NAME_GNOMECHARMER_TOGGLE_BAR = "Toggle bar"
	function GnomeCharmer_ToggleBar()
		GnomeCharmerDB.barShow = not GnomeCharmerDB.barShow
		barOnEvent()
		DEFAULT_CHAT_FRAME:AddMessage(("GnomeCharmer bar toggled %s."):format(GnomeCharmerDB.barShow and "On" or "Off"))
	end

	createBar()
	bar:Hide()
end

-----------------------------------------------------------------------
-- Keep icons dimmed when used and full when available.
--

local function updateAlpha()
	for i = 1, 8 do
		local a = iconsUsed[i] and 0.3 or 1
		if radialIcons and radialIcons[i] then radialIcons[i]:SetAlpha(a) end
		if barIcons and barIcons[i] then barIcons[i]:SetAlpha(a) end
	end
end

hooksecurefunc("SetRaidTarget", function(unit, id)
	if id > 0 and validate(unit) then
		local uid = UnitGUID(unit)
		if uid and iconsUsed[id] == uid then
			iconsUsed[id] = nil
		else
			iconsUsed[id] = uid or true
		end
	elseif id == 0 then
		local uid = UnitGUID(unit)
		if uid then
			for k, v in pairs(iconsUsed) do
				if v == uid then
					iconsUsed[k] = nil
					break
				end
			end
		end
	end
	updateAlpha()
end)

-- Mark icons used on dead mobs as available, except if it's a friendly player.
local events = CreateFrame("Frame")
events:SetScript("OnEvent", function(self, event, ...)
	if event == "ADDON_LOADED" and arg1 == "GnomeCharmer" then
		if not GnomeCharmerDB then GnomeCharmerDB = {
			barShow = true,
			barAlpha = 1,
		} end
	else
		local target = "target"
		if event == "UPDATE_MOUSEOVER_UNIT" then
			target = "mouseover"
		end
		local icon = GetRaidTargetIndex(target)
		if icon then
			if UnitIsDeadOrGhost(target) and not UnitIsPlayer(target) then
				iconsUsed[icon] = nil
			else
				iconsUsed[icon] = true
			end
			updateAlpha()
		end
	end
end)
events:RegisterEvent("ADDON_LOADED")
--events:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
--events:RegisterEvent("PLAYER_TARGET_CHANGED")

